#include <sqlextract/filter/filter_core.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sqlextract/util.h>
#include <sqlextract/filter/arbval.h>

struct filter_t {
	void (*process)(struct arbval_t *, void *, void *);
	void (*destroy)(void *);
	void *storage;
	struct arbval_t *result;
};

struct filter_t *filter_create(
		void (*process)(struct arbval_t *, void *, void *),
		void (*destroy)(void *), void *storage) {
	struct filter_t *flt;
	flt = lf_malloc(sizeof(struct filter_t));
	if (flt == NULL) {
		(*destroy)(storage);
		return NULL;
	}

	flt->process = process;
	flt->destroy = destroy;
	flt->storage = storage;
	flt->result = arbval_create_null();

	return flt;
}

void filter_destroy(struct filter_t *flt) {
	if (flt == NULL)
		return;
	if (flt->destroy)
		(*flt->destroy)(flt->storage);
	arbval_destroy(flt->result);
	free(flt);
}

struct arbval_t *filter_process(struct filter_t *flt, void *object) {
	(*flt->process)(flt->result, object, flt->storage);
	return flt->result;
}
